from PySide2.QtWidgets import (
    QApplication,
    QDialog, QMenuBar, QVBoxLayout, QHBoxLayout, QSplitter,
    QWidget,
    QPushButton,
    QTableView,
    QAbstractItemView,
    QSlider,
    QLineEdit,
    QLabel,
    QComboBox,
    QFileDialog,  # keep it in
)

from PySide2.QtGui import (
    QStandardItemModel,
    QStandardItem,
)

from PySide2.QtCore import Qt
